﻿Ext.require('App.Const.Finances');
Ext.define('App.Control.Kivii.Finances.Settlement.MultiForm', {
    extend: 'System.Control.EntityForm',
    xtype: 'App.Control.Kivii.Finances.Settlement.MultiForm',
    entityType: 'Kivii.Finances.Entities.Settlement',
    scrollable: true,
    layout: {
        type: 'responsivecolumn',
        states: {
            small: 600,
            large: 0
        }
    },
    title: '批量结算',
    iconCls: 'x-fa fa-newspaper-o',
    viewModel: {
        data: {
            entity: {
                Metadata: {},
                Kvid: null,
                Amount: 0,
                AmountPlan: 0,
                AmountPayment: 0,
                BizType: 'Kivii.Finances.Entities.Settlement',
                PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
                PayeeName: KiviiContext.CurrentMember.OrganizationName,
                OperatorName: KiviiContext.CurrentMember.FullName,
                OperatorKvid: KiviiContext.CurrentMember.Kvid,
                Currency: 'CNY'
            }
        },
        formulas: {
            amountChineseBig: function(get) {
                let _amount = get('entity.Amount');
                return KiviiContext.Const.FinancesFunctionsMoneyToChineseBig(_amount);
            }
        }
    },
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: ['->', {
            text: '批量结算',
            iconCls: 'x-fa fa-link',
            iconAlign: 'left',
            handler: '_onSaveEntity',
            bind: {
                disabled: '{!formIsValid}'
            }
        }]
    }],
    defaults: {
        xtype: 'textfield',
        labelWidth: 60,
        labelAlign: 'right',
        responsiveCls: 'big-50 small-100'
    },
    items: [{
        xtype: 'windowSelector',
        fieldLabel: '付款单位',
        responsiveCls: 'big-60 small-100',
        editable: true,
        keyField: 'Kvid',
        valueField: 'FullName',
        window: {
            type: 'App.Control.Kivii.Finances.Settlement.Customer.Selector'
        },
        allowBlank: false,
        bind: {
            key: '{entity.PayerKvid}',
            value: '{entity.PayerName}',
            readOnly: '{entity.Kvid}'
        }
    }, {
        xtype: 'windowSelector',
        fieldLabel: '收款单位',
        responsiveCls: 'big-40 small-100',
        readOnly: true,
        editable: false,
        keyField: 'Kvid',
        valueField: 'FullName',
        window: {
            type: 'App.Control.Kivii.Finances.Settlement.Customer.Selector'
        },
        allowBlank: false,
        bind: {
            key: '{entity.PayeeKvid}',
            value: '{entity.PayeeName}'
        }
    }, {
        xtype: 'numberfield',
        fieldLabel: '总&nbsp;&nbsp;原&nbsp;价',
        hideTrigger: true,
        minValue: 0,
        responsiveCls: 'big-30 small-100',
        step: 0.1,
        allowBlank: false,
        readOnly: true,
        bind: {
            value: '{entity.AmountPlan}'
        }
    }, {
        xtype: 'numberfield',
        fieldLabel: '总折后价',
        hideTrigger: true,
        minValue: 0,
        responsiveCls: 'big-30 small-100',
        step: 0.1,
        allowBlank: false,
        readOnly: true,
        bind: {
            value: '{entity.Amount}'
        }
    }, {
        xtype: 'numberfield',
        fieldLabel: '已收款额',
        hideTrigger: true,
        minValue: 0,
        responsiveCls: 'big-20 small-100',
        step: 0.1,
        allowBlank: false,
        readOnly: true,
        bind: {
            value: '{entity.AmountPayment}'
        }
    }, {
        xtype: 'categoryField',
        fieldLabel: '货币单位',
        allowBlank: false,
        readOnly: true,
        responsiveCls: 'big-20 small-100',
        internalCode: 'Currency',
        bind: {
            value: '{entity.Currency}'
        }
    }, {
        responsiveCls: 'big-60 small-100',
        fieldLabel: '金额大写',
        readOnly: true,
        bind: {
            value: '{amountChineseBig}'
        }
    }, {
        fieldLabel: '操&nbsp;&nbsp;作&nbsp;人',
        responsiveCls: 'big-40 small-100',
        allowBlank: false,
        readOnly: true,
        bind: '{entity.OperatorName}'
    }, {
        xtype: 'textarea',
        fieldLabel: '备&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;注',
        responsiveCls: 'big-100 small-100',
        bind: {
            value: '{entity.Remark}',
            readOnly: '{entity.Kvid}'
        }
    }, {
        xtype: 'fieldset',
        title: '结算项',
        collapsible: true,
        collapsed: false,
        scrollable: true,
        responsiveCls: 'big-100 small-100',
        items: [{
            xtype: 'grid',
            reference: 'details',
            store: {
                pageSize: 10000,
                data: [],
                groupField: 'Type',
                autoLoad: false
            },
            features: [{
                ftype: 'groupingsummary',
                groupHeaderTpl: [
                    '<div>{name:this.formatName} ({rows.length} 条)</div>',
                    {
                        formatName: function(name) {
                            if (name === "Contract") {
                                return "合同费";
                            }
                            if (name === "Report") {
                                return "报告费";
                            }
                            if (name === "Service") {
                                return "服务费";
                            }
                            if (name === "Express") {
                                return "快递费";
                            }
                            return name;
                        }
                    }
                ]
            }],
            dockedItems: [{
                xtype: 'toolbar',
                dock: 'top',
                overflowHandler: 'menu',
                items: [{
                    margin: '0 3 -10 3',
                    xtype: 'displayfield',
                    value: '结算明细',
                    fieldStyle: {
                        fontSize: '20px'
                    }
                }]
            }],
            columns: {
                defaults: {
                    menuDisabled: true,
                    sortable: false
                },
                items: [{
                    xtype: 'rownumberer'
                }, {
                    xtype: 'actioncolumn',
                    align: 'center',
                    text: '操作',
                    items: [{
                        //     text: '编辑',
                        //     iconCls: 'x-fa fa-pencil-square-o',
                        //     handler: '_onEditDetail'
                        // }, '->', {
                        text: '移除',
                        iconCls: 'x-fa fa-trash-o',
                        handler: '_onRemoveDetail'
                    }],
                    width: 80
                }, {
                    dataIndex: 'BizId',
                    text: '所属业务',
                    width: 170,
                    // sortable: true,
                    summaryType: 'sum',
                    summaryRenderer: function(value, summaryData, dataIndex) {
                        return Ext.String.format('合计');
                    }
                }, {
                    dataIndex: 'OwnerName',
                    text: '所属部门',
                    width: 120
                }, {
                    dataIndex: 'OperateTime',
                    text: '操作日期',
                    width: 100,
                    // sortable: true,
                    renderer: Ext.util.Format.dateRenderer('Y-m-d')
                }, {
                    dataIndex: 'AmountPlan',
                    align: 'right',
                    text: '原金额',
                    width: 100,
                    // renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
                    summaryType: 'sum',
                    summaryRenderer: function(value, summaryData, dataIndex) {
                        return Ext.String.format('￥{0}', value.toFixed(2));
                    }
                }, {
                    dataIndex: 'Amount',
                    align: 'right',
                    text: '结算金额',
                    width: 100,
                    // renderer: KiviiContext.Const.FinancesFunctionsMoneyToFmoney,
                    summaryType: 'sum',
                    summaryRenderer: function(value, summaryData, dataIndex) {
                        return Ext.String.format('￥{0}', value.toFixed(2));
                    }
                }, {
                    dataIndex: 'Type',
                    text: '类型',
                    width: 90,
                    renderer: '_renderType'
                }, {
                    dataIndex: 'OperatorName',
                    text: '操作人',
                    width: 90
                }, {
                    dataIndex: 'Summary',
                    text: '摘要',
                    width: 200
                }, {
                    dataIndex: 'Remark',
                    text: '备注',
                    minWidth: 100,
                    flex: 1
                }]
            }
        }]
    }],
    setOwner: function(_entities) {
        let _view = this;
        let _vm = _view.getViewModel();
        let _refs = _view.getReferences();
        let _detailsStore = _refs.details.getStore();
        let _controller = _view.getController();
        let _entity = {
            Metadata: {},
            Kvid: null,
            Amount: 0,
            AmountPlan: 0,
            AmountPayment: 0,
            BizType: 'Kivii.Finances.Entities.Settlement',
            PayeeKvid: KiviiContext.CurrentMember.OrganizationKvid,
            PayeeName: KiviiContext.CurrentMember.OrganizationName,
            OperatorName: KiviiContext.CurrentMember.FullName,
            OperatorKvid: KiviiContext.CurrentMember.Kvid,
            Currency: 'CNY'
        };
        for (let _i = 0; _i < _entities.length; _i++) {
            let _detail = _entities[_i];
            _detailsStore.add(_detail);
            _entity.PayerName = _detail.PayerName;
            _entity.PayerKvid = _detail.PayerKvid;
            _entity.BizType = _detail.BizType;
        }
        _vm.set('entity', _entity);
        _controller._reCalculateAmount();
    },
    controller: {
        _reCalculateAmount: function() {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _refs = _controller.getReferences();
            let _details = _refs.details.getStore().getData().items;
            let _amount = 0;
            let _amountPlan = 0;
            for (let _i = 0; _i < _details.length; _i++) {
                let _record = _details[_i].data;
                _amount += parseFloat(_record.Amount);
                _amountPlan += parseFloat(_record.AmountPlan);
            }
            _vm.set('entity.AmountPlan', parseFloat(_amountPlan.toFixed(2)));
            _vm.set('entity.Amount', parseFloat(_amount.toFixed(2)));
        },
        _renderType: function(_value, _meta, _record, _rowIndex) {
            if (_value === "Contract") {
                return "合同费";
            }
            if (_value === "Report") {
                return "报告费";
            }
            if (_value === "Service") {
                return "服务费";
            }
            if (_value === "Express") {
                return "快递费";
            }
            return _value;
        },
        _onRemoveDetail: function(_ctrl, _recIndex, _cellIndex, _item, _e, _record) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _kvid = _vm.data.entity.Kvid;
            if (_kvid !== null&&_kvid!==undefined) {
                Ext.toast("已结算项目,无法移除");
                return;
            }
            _record.drop();
            _controller._reCalculateAmount();
        },
        _onSaveEntity: function(_ctrl) {
            let _controller = this;
            let _vm = _controller.getViewModel();
            let _view = _controller.getView();
            let _refs = _controller.getReferences();
            let _item = _vm.data.entity;
            let _details = _refs.details.getStore().getData().items;
            if (_details.length <= 0) {
                Ext.toast("无结算项,无法新增!");
                return;
            }
            let _entities = [];
            for (let _i = 0; _i < _details.length; _i++) {
                let _record = _details[_i].data;
                _record.PayerKvid = _item.PayerKvid;
                _record.PayerName = _item.PayerName;
                _record.Remark = _item.Remark;
                _entities.push(_record);
            }
            let _jsonData = {};
            _jsonData.EntityType = _item.BizType;
            _jsonData.Items = _entities;
            _ctrl.setDisabled(true);
            Ext.Ajax.request({
                url: '/Restful/' + _view.entityType + '/Create.json',
                jsonData: _jsonData,
                success: function(_response) {
                    let _rtns = Ext.decode(_response.responseText);
                    Ext.toast('结算成功!');
                    _ctrl.setDisabled(false);
                    _view.fireEvent('entityCreated', _view, _rtns.Results);
                },
                failure: function(_response) {
                    Ext.toast('结算失败!');
                    _ctrl.setDisabled(false);
                }
            });
        }
    }
});