﻿Ext.require('App.Const.Finances');
Ext.define('App.View.Kivii.Finances.Statistic.Settlement', {
    extend: 'System.Control.EntityList',
    xtype: 'App.View.Kivii.Finances.Statistic.Settlement',
    entityType: 'Kivii.Finances.Entities.Settlement',
    dataSource: '/Restful/Kivii.Finances.Entities.Settlement/Statistic.json',
    globalFilter: {},
    filter: {
        BeginTime: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
        EndTime: new Date(new Date().getFullYear(), new Date().getMonth(), 1),
        // TimeType: 'OperateTime',
        Payed: true
    },
    header: true,
    layout: 'fit',
    title: '报告结算统计',
    iconCls: 'x-fa fa-calculator',
    viewModel: {
        data: {
            // AmountTask: 0,
            // AmountTaskTotal: 0,
            // AmountTaskDiscount: 0,
            // AmountTaskDiscountTotal: 0,
            // AmountPackage: 0,
            // AmountCalculation: 0,
            // AmountReport: 0,
            // AmountReportChange: 0,
            // AmountPayment: 0,
            // DiscountRate: 0,
            // reportQuantity: 0,
            // xReportQuantity: 0,
            _selection: null,
            lastOwnerKvid: null
        }
    },
    bind: {
        selection: '{_selection}'
    },
    store: {
        autoLoad: true,
        pageSize: 800,
        sorters: [{
            property: 'CreateTime',
            direction: 'ASC'
        }]
    },
    features: [{
        ftype: 'summary'
    }],
    dockedItems: [{
        xtype: 'toolbar',
        dock: 'top',
        overflowHandler: 'menu',
        items: [{
            xtype: 'combo',
            queryMode: 'local',
            displayField: 'name',
            valueField: 'payed',
            value: true,
            width: 100,
            editable: false,
            readOnly: false,
            filterPickList: true,
            reference: 'payed',
            store: {
                fields: ['name', 'payed'],
                data: [{
                    name: '已收费',
                    payed: true
                }, {
                    name: '未收费',
                    payed: false
                }, {
                    name: '全部',
                    payed: null
                }]
            },
            listeners: {
                select: '_onSelectPayed'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '时间范围  开始',
            format: 'Y-m-d',
            labelWidth: 90,
            reference: 'startTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth() - 1, 1),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'datefield',
            fieldLabel: '结束',
            format: 'Y-m-d',
            labelWidth: 30,
            reference: 'endTime',
            editable: false,
            value: new Date(new Date().getFullYear(), new Date().getMonth(), 0),
            // value: new Date(),
            listeners: {
                select: '_onSelectMonth'
            }
        }, {
            xtype: 'searchField',
            flex: 1,
            labelWidth: 60,
            queryField: 'PayerName,BizId',
            emptyText: '模糊搜索付款单位、所属报告'
        }, {
            text: '导出报表',
            iconCls: 'x-fa fa-download',
            handler: '_onDownloadExcel'
        }]
    }, {
        xtype: 'pagingtoolbar',
        dock: 'bottom',
        displayInfo: true,
        items: [{
            margin: '0 0 -10 3',
            xtype: 'displayfield',
            bind: {
                value: '报告费: {AmountSettlement}&nbsp;&nbsp;已收费: {AmountPayment}&nbsp;&nbsp;未收费: {AmountUnpaid}&nbsp;&nbsp;已作废: {AmountOffset}&nbsp;&nbsp;批次:{QuantityPayment}/{QuantitySettlement}(作废:{QuantityOffset})'
            },
            fieldStyle: {
                fontSize: '14px'
            }
        }]
    }],
    columns: {
        defaults: {
            menuDisabled: true,
            sortable: false
        },
        items: [{
            xtype: 'rownumberer'
        }, {
            dataIndex: 'BizId',
            text: '所属报告',
            width: 170,
            sortable: true,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('合计');
            }
        }, {
            dataIndex: 'PayerName',
            text: '付款单位',
            width: 150,
            sortable: true
        }, {
            dataIndex: 'OperateTime',
            text: '委托日期',
            width: 100,
            sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'CreateTime',
            text: '结算日期',
            width: 100,
            sortable: true,
            renderer: Ext.util.Format.dateRenderer('Y-m-d')
        }, {
            dataIndex: 'Amount',
            align: 'right',
            text: '合计',
            width: 90,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'AmountPayment',
            align: 'right',
            text: '已收款',
            width: 90,
            summaryType: 'sum',
            summaryRenderer: function(value, summaryData, dataIndex) {
                return Ext.String.format('￥{0}', value.toFixed(2));
            }
        }, {
            dataIndex: 'Metadata.AmountTaskDiscountTotal',
            text: '折后费',
            width: 90
        }, {
            dataIndex: 'Metadata.AmountPackage',
            text: '套餐费',
            width: 90
        }, {
            dataIndex: 'Metadata.CustomerOrgName',
            text: '委托单位',
            width: 120
        }, {
            dataIndex: 'Metadata.CustomerContactName',
            text: '送样人',
            width: 70
        }, {
            dataIndex: 'Metadata.ProductOrgName',
            text: '生产单位',
            width: 120
        }, {
            dataIndex: 'Metadata.ProductContactName',
            text: '送样人',
            width: 70
        }, {
            dataIndex: 'Metadata.AgreementName',
            text: '合同',
            width: 70
        }, {
            dataIndex: 'Metadata.BatchNumber',
            text: '批号货号',
            width: 100
        }, {
            dataIndex: 'Summary',
            text: '所属附件',
            width: 150
        }, {
            dataIndex: 'OperatorName',
            text: '结算人',
            width: 70
        }, {
            dataIndex: 'Remark',
            text: '备注',
            minWidth: 100,
            flex: 1
        }]
    },
    listeners: {
        activate: '_onActivate',
        viewready: '_viewReady'
    },
    controller: {
        _onActivate: function(_ctrl, _eOpts) {
            this.getView().reload();
        },
        _onClearSelections: function() {
            this.getView().clearSelections();
        },
        _viewReady: function() {
            var _me = this;
            var _view = _me.getView();
            var _vm = _me.getViewModel();
            var _store = _view.getStore();
            _store.on('load', function(_this, _records, _successful, _operation, _eOpts) {
                let _rtns = Ext.decode(_operation.getResponse().responseText);
                _vm.set('AmountSettlement', _rtns.TotalAmount.toFixed(2));
                _vm.set('AmountPayment', _rtns.TotalAmountPayment.toFixed(2));
                _vm.set('AmountUnpaid', _rtns.TotalAmountUnpaid.toFixed(2));
                _vm.set('AmountOffset', _rtns.TotalAmountOffset.toFixed(2));
                _vm.set('Quantity', _rtns.TotalQuantity.toFixed(0));
                _vm.set('QuantityPayment', _rtns.TotalQuantityPayment.toFixed(0));
                _vm.set('QuantityUnpaid', _rtns.TotalQuantityUnpaid.toFixed(0));
                _vm.set('QuantitySettlement', _rtns.TotalQuantity.toFixed(0));
                _vm.set('QuantityOffset', _rtns.TotalQuantityOffset.toFixed(0));
            });
        },
        _onSelectPayed: function(_ctrl, _record) {
            var _controller = this,
                _view = _controller.getView(),
                _vm = _controller.getViewModel();
            delete _view.filter.Payed;
            _view.filter.Payed = _record.data.payed;
            _view.reload();
        },
        _onSelectMonth: function(_ctrl, _time, _eOpts) {
            var _controller = this;
            var _view = _controller.getView();
            var _refs = _controller.getReferences();
            var _startTime = _refs.startTime.value;
            var _endTime = _refs.endTime.value || new Date();
            if (_startTime === null) {
                Ext.toast("请选择开始时间!");
                return;
            }
            if (_startTime > _endTime) {
                Ext.toast("开始日期不能大于结束日期!");
                return;
            }
            delete _view.filter.BeginTime;
            delete _view.filter.EndTime;
            _view.filter.BeginTime = new Date(_startTime.getFullYear(), _startTime.getMonth(), _startTime.getDate());
            _view.filter.EndTime = new Date(_endTime.getFullYear(), _endTime.getMonth(), _endTime.getDate() + 1);
            _view.reload();
        },
        _onDownloadExcel: function(_ctrl) {
            var _controller = this;
            var _vm = _controller.getViewModel();
            var _view = _controller.getView();
            var _refs = _controller.getReferences();
            var _filter = _view.getFilter();
            var _globalFilter = _view.getGlobalFilter();
            var _searchFilter = _view.getSearchFilter();
            let _selector = Ext.create('App.Control.Kivii.Finances.General.Template.Selector');

            var _year = _refs.startTime.value.getFullYear();
            var _month = _refs.startTime.value.getMonth() + 1;

            var _name = '[' + _year + '年' + _month + '月份]';
            var _payed = _refs.payed.getDisplayValue();
            if (_payed !== "") _name += '_' + _payed;

            var _pageSize = _view.getStore().getPageSize();
            var _currentPage = _view.getStore().currentPage;
            var _totalPage = Math.ceil(_view.getStore().totalCount / _pageSize);

            var _win = Ext.create('Ext.window.Window', {
                layout: 'fit',
                title: '选择模板',
                iconCls: 'x-fa fa-money',
                iconAlign: 'left',
                autoScroll: true,
                width: 700,
                height: 350,
                modal: true,
                resizable: false,
                items: [_selector]
            });
            _win.show();
            _selector.setFilter({
                FolderPath: '/Finances/Settlement/Files',
                Tags: 'Settlement'
            });
            _selector.on('entitySelected', function(_ctrl, _entity) {
                _win.close();
                var _iframe = document.createElement("iframe");
                _iframe.style.display = "none";
                var _form = document.createElement('form');
                for (var _globalKey in _globalFilter) {
                    var _globalValue = document.createElement('input');
                    _globalValue.type = 'hidden';
                    _globalValue.name = _globalKey;
                    _globalValue.value = _globalFilter[_globalKey];
                    if ((typeof _globalFilter[_globalKey] === 'object') && _globalFilter[_globalKey].constructor === Date) {
                        var _globalDate = _globalFilter[_globalKey].getFullYear() + '-' + (_globalFilter[_globalKey].getMonth() + 1) + '-' + _globalFilter[_globalKey].getDate();
                        _globalValue.value = _globalDate;
                    }
                    _form.appendChild(_globalValue);
                }
                for (var _filterKey in _filter) {
                    var _filterValue = document.createElement('input');
                    _filterValue.type = 'hidden';
                    _filterValue.name = _filterKey;
                    _filterValue.value = _filter[_filterKey];
                    if ((typeof _filter[_filterKey] === 'object') && _filter[_filterKey].constructor === Date) {
                        var _filterDate = _filter[_filterKey].getFullYear() + '-' + (_filter[_filterKey].getMonth() + 1) + '-' + _filter[_filterKey].getDate();
                        _filterValue.value = _filterDate;
                    }
                    _form.appendChild(_filterValue);
                }
                for (var _searchKey in _searchFilter) {
                    var _searchValue = document.createElement('input');
                    _searchValue.type = 'hidden';
                    _searchValue.name = _searchKey;
                    _searchValue.value = _searchFilter[_searchKey];
                    if ((typeof _searchFilter[_searchKey] === 'object') && _searchFilter[_searchKey].constructor === Date) {
                        var _searchDate = _searchFilter[_searchKey].getFullYear() + '-' + (_searchFilter[_searchKey].getMonth() + 1) + '-' + _searchFilter[_searchKey].getDate();
                        _searchValue.value = _searchDate;
                    }
                    _form.appendChild(_searchValue);
                }

                var _view = document.createElement('input');
                _view.type = 'hidden';
                _view.name = 'View';
                _view.value = _entity[0].Path;
                _form.appendChild(_view);

                var _fileName = document.createElement('input');
                _fileName.type = 'hidden';
                _fileName.name = '__fileName';
                _fileName.value = _name + '(第' + _currentPage + '页,共' + _totalPage + '页)';
                _form.appendChild(_fileName);

                var _skip = document.createElement('input');
                _skip.type = 'hidden';
                _skip.name = 'Skip';
                _skip.value = (_currentPage - 1) * _pageSize;
                _form.appendChild(_skip);

                var _take = document.createElement('input');
                _take.type = 'hidden';
                _take.name = 'Take';
                _take.value = _pageSize;
                _form.appendChild(_take);

                var _orderBy = document.createElement('input');
                _orderBy.type = 'hidden';
                _orderBy.name = 'OrderBy';
                _orderBy.value = 'OperateTime';
                _form.appendChild(_orderBy);

                _iframe.appendChild(_form);
                document.body.appendChild(_iframe);
                _form.method = 'GET';
                _form.action = '/Restful/Kivii.Finances.Entities.Settlement/Statistic.xlsx';
                _form.submit();
                document.body.removeChild(_iframe);
            });

        }
    }
});