﻿using Kivii.Finances.Entities;
using Kivii.Linq;
using Kivii.Web;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Kivii.Finances.Transforms
{
    [Api(Description = "生成到账流水")]
    [RequiresAnyRole(SystemRoles.Administrator,SystemRoles.Manager,SystemRoles.Configuration)]
    public class PaymentAccept : RestfulExecution<Payment>
    {
        public Payment Item { get; set; }

        public List<Payment> Items { get; set; }

        //[ApiMember(Description = "认领类型,默认为Department,Organization,Member")]
        //public OwnerType OwnerType { get; set; } = OwnerType.Organization;

        public override object OnExecution(IRequest req, IResponse res)
        {
            (Item == null && Items.IsNullOrEmpty()).ThrowIfTrue("Need Item Or Items!");
            if (Items.IsNullOrEmpty()) Items = new List<Payment>();
            if (Item != null) Items.Add(Item);
            if (Items.Exists(o => o.Currency != CurrencyUnit.CNY)) throw new Exception("暂仅支持生成人民币货币单位流水!");
            ////这两步验证用于验证针对当前登录组织是收款还是付款
            //if (Items.Exists(o => o.PayerAccountKvid == Guid.Empty && o.PayeeAccountKvid == Guid.Empty)) throw new Exception("收付款账户不能同时为空!");
            //if (Items.Exists(o => o.PayerAccountKvid != Guid.Empty && o.PayeeAccountKvid != Guid.Empty)) throw new Exception("收付款账户不能同时指定!");

            var typeBanks = Items.Where(o => o.Type == PaymentType.Bank).ToList();
            var typeCashs = Items.Where(o => o.Type == PaymentType.Cash).ToList();
            var typePoss = Items.Where(o => o.Type == PaymentType.Pos).ToList();

            var serialNumbers = Items.ConvertAll(o => o.SerialNumber);
            serialNumbers.RemoveAll(o => o.IsNullOrEmpty());
            var conn = KiviiContext.GetOpenedDbConnection<Payment>();
            var accountBalance = CurrencyUnit.CNY.GetBalanceAccount();

            List<Payment> existPayments = null;
            if (!serialNumbers.IsNullOrEmpty()) existPayments = conn.Select<Payment>(o => Sql.In(o.SerialNumber, serialNumbers));
            var allAccountDetails = new List<AccountDetail>();
            var allAcceptPayments = new List<Payment>();
            var allAcceptPays = new List<Pay>();

            //生成银行流水
            if (!typeBanks.IsNullOrEmpty())
            {
                foreach (var item in typeBanks)
                {
                    #region 验证是否为重复项
                    if (!existPayments.IsNullOrEmpty())
                    {
                        var exist = existPayments.FirstOrDefault(o => o.SerialNumber == item.SerialNumber);
                        if (exist != null) continue;
                    }
                    else
                    {
                        var exist = conn.Single<Payment>(o => o.PayerName == item.PayerName && o.OperateTime == item.OperateTime && o.Amount == item.Amount && o.RootKvid == o.Kvid);
                        if (exist != null) continue;
                    }
                    #endregion
                    if (item.Amount <= 0) continue;
                    //Account account = null;
                    //if (item.PayerAccountKvid != Guid.Empty) account = accounts.FirstOrDefault(o => o.Kvid == item.PayerAccountKvid);
                    //if (item.PayeeAccountKvid != Guid.Empty) account = accounts.FirstOrDefault(o => o.Kvid == item.PayeeAccountKvid);
                    //if (account == null) continue;
                    var itemKvid = Guid.NewGuid();
                    item.AmountInvoice = 0;
                    item.AmountSplited = 0;
                    item.AmountUsed = 0;

                    item.OffsetKvid = Guid.Empty;
                    item.BizId = string.Empty;
                    item.BizType = string.Empty;
                    item.BizKvid = Guid.Empty;
                    item.Kvid = itemKvid;
                    item.RootKvid = item.Kvid;
                    item.ParentKvid = Guid.Empty;

                    item.PayerAccountKvid = accountBalance.Kvid;
                    item.PayerAccountName = item.PayerAccountName.IsNullOrEmpty() ? accountBalance.Name : item.PayerAccountName;
                    item.PayerAccountOwnerName = item.PayerAccountOwnerName.IsNullOrEmpty() ? accountBalance.OwnerName : item.PayerAccountOwnerName;
                    item.PayerAccountSerialNumber = item.PayerAccountSerialNumber.IsNullOrEmpty() ? accountBalance.SerialNumber : item.PayerAccountSerialNumber;

                    item.PayeeAccountKvid = Guid.Empty;
                    if (item.PayeeName.IsNullOrEmpty()) item.PayeeName = KiviiContext.CurrentMember.OrganizationName;
                    item.OperateTime = (item.OperateTime == DateTime.MinValue ? DateTime.Now : item.OperateTime);
                    item.OperatorName = KiviiContext.CurrentMember.FullName;
                    item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                    var accountDetail = item.Accept(out Pay pay);
                    allAcceptPayments.Add(item);
                    allAccountDetails.Add(accountDetail);
                    allAcceptPays.Add(pay);
                }
            }

            //生成现金流水
            if (!typeCashs.IsNullOrEmpty())
            {
                var cashAccount = CurrencyUnit.CNY.GetPersonalAccount(AccountType.Cash);
                if (cashAccount == null) throw new Exception("未找到当前登录人的现金账户!");
                foreach (var item in typeCashs)
                {
                    #region 验证是否为重复项
                    if (!existPayments.IsNullOrEmpty())
                    {
                        var exist = existPayments.FirstOrDefault(o => o.SerialNumber == item.SerialNumber);
                        if (exist != null) continue;
                    }
                    else
                    {
                        var exist = conn.Single<Payment>(o => o.PayerName == item.PayerName && o.OperateTime == item.OperateTime && o.Amount == item.Amount && o.RootKvid == o.Kvid);
                        if (exist != null) continue;
                    }
                    #endregion
                    if (item.Amount <= 0) continue;
                    var itemKvid = Guid.NewGuid();
                    item.AmountInvoice = 0;
                    item.AmountSplited = item.Amount;
                    item.AmountUsed = 0;

                    item.OffsetKvid = Guid.Empty;
                    item.BizId = string.Empty;
                    item.BizType = string.Empty;
                    item.BizKvid = Guid.Empty;
                    item.Kvid = itemKvid;
                    item.RootKvid = item.Kvid;
                    item.ParentKvid = Guid.Empty;

                    item.PayerAccountKvid = cashAccount.Kvid;
                    item.PayerAccountName = item.PayerAccountName.IsNullOrEmpty() ? cashAccount.Name : item.PayerAccountName;
                    item.PayerAccountOwnerName = item.PayerAccountOwnerName.IsNullOrEmpty() ? cashAccount.OwnerName : item.PayerAccountOwnerName;
                    item.PayerAccountSerialNumber = item.PayerAccountSerialNumber.IsNullOrEmpty() ? cashAccount.SerialNumber : item.PayerAccountSerialNumber;

                    item.PayeeAccountKvid = Guid.Empty;
                    if (item.PayeeName.IsNullOrEmpty()) item.PayeeName = KiviiContext.CurrentMember.OrganizationName;
                    item.OperateTime = (item.OperateTime == DateTime.MinValue ? DateTime.Now : item.OperateTime);
                    item.OperatorName = KiviiContext.CurrentMember.FullName;
                    item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                    var accountDetail = item.Accept(out Pay pay);
                    allAcceptPayments.Add(item);
                    allAccountDetails.Add(accountDetail);
                    allAcceptPays.Add(pay);
                }
            }
            //生成Pos流水
            if (!typePoss.IsNullOrEmpty())
            {
                var posAccount = CurrencyUnit.CNY.GetPersonalAccount(AccountType.Pos);
                if (posAccount == null) throw new Exception("未找到当前登录人的刷卡账户!");
                foreach (var item in typePoss)
                {
                    #region 验证是否为重复项
                    if (!existPayments.IsNullOrEmpty())
                    {
                        var exist = existPayments.FirstOrDefault(o => o.SerialNumber == item.SerialNumber);
                        if (exist != null) continue;
                    }
                    else
                    {
                        var exist = conn.Single<Payment>(o => o.PayerName == item.PayerName && o.OperateTime == item.OperateTime && o.Amount == item.Amount && o.RootKvid == o.Kvid);
                        if (exist != null) continue;
                    }
                    #endregion
                    if (item.Amount <= 0) continue;
                    var itemKvid = Guid.NewGuid();
                    item.AmountInvoice = 0;
                    item.AmountSplited = item.Amount;
                    item.AmountUsed = 0;

                    item.OffsetKvid = Guid.Empty;
                    item.BizId = string.Empty;
                    item.BizType = string.Empty;
                    item.BizKvid = Guid.Empty;
                    item.Kvid = itemKvid;
                    item.RootKvid = item.Kvid;
                    item.ParentKvid = Guid.Empty;

                    item.PayerAccountKvid = posAccount.Kvid;
                    item.PayerAccountName = item.PayerAccountName.IsNullOrEmpty() ? posAccount.Name : item.PayerAccountName;
                    item.PayerAccountOwnerName = item.PayerAccountOwnerName.IsNullOrEmpty() ? posAccount.OwnerName : item.PayerAccountOwnerName;
                    item.PayerAccountSerialNumber = item.PayerAccountSerialNumber.IsNullOrEmpty() ? posAccount.SerialNumber : item.PayerAccountSerialNumber;

                    item.PayeeAccountKvid = Guid.Empty;
                    if (item.PayeeName.IsNullOrEmpty()) item.PayeeName = KiviiContext.CurrentMember.OrganizationName;
                    item.OperateTime = (item.OperateTime == DateTime.MinValue ? DateTime.Now : item.OperateTime);
                    item.OperatorName = KiviiContext.CurrentMember.FullName;
                    item.OperatorKvid = KiviiContext.CurrentMember.Kvid;
                    var accountDetail = item.Accept(out Pay pay);
                    allAcceptPayments.Add(item);
                    allAccountDetails.Add(accountDetail);
                    allAcceptPays.Add(pay);
                }
            }

            var rtns = new RestfulCreateResponse<Payment>();
            rtns.Results = new List<Payment>();

            var trans = conn.OpenTransaction();

            try
            {
                if (!allAcceptPayments.IsNullOrEmpty())
                {
                    foreach (var item in allAcceptPayments)
                    {
                        conn.Insert(item);
                        rtns.Results.Add(item);
                    }
                }
                if (!allAccountDetails.IsNullOrEmpty()) allAccountDetails.ForEach(o => o.Insert(conn));
                if (!allAcceptPays.IsNullOrEmpty())
                {
                    foreach (var item in allAcceptPays)
                    {
                        conn.Insert(item);
                    }
                }
                trans.Commit();
            }
            catch (Exception ex)
            {
                trans.Rollback();
                throw ex;
            }

            return rtns;
        }
    }
}
